/*

					Logic Scope AVR

			Copyrights 2009,  Yoshi Hiroshi


*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "logicScope.h"
#include "logCommand.h"
#include "uart.h"
#include "commMsg.h"

unsigned char volatile dummy;

//**********************************************************
//			Standard Logging Function
//**********************************************************
//	Calling Parameters
//		buffer:
//		interval:
//			 20uS	  20000	   50K samples/sec
//			 50uS	  50000	   20K samples/sec
//			100uS	 100000	   10K samples/sec
//			200uS	 200000	    5K samples/sec
//			500uS	 500000	    2K samples/sec
//			  1mS	1000000     1K samples/sec
//		mask:
//		pattern:
//----------------------------------------------------------

void normalLog()
{
	unsigned char temp;
	lcb.dataReady = 0;
	lcb.logCounter = 1;
	lcb.busyCounterMax = 40000 / (lcb.period / 1000);
	lcb.busyCounter = 0;
#if defined (__AVR_ATmega644P__)
	NORMAL_LOG_OCRA_SET(((lcb.period / 1000) * 20) - 1);
	NORMAL_LOG_TCNT_SET(69);
#endif
#if defined (__AVR_ATmega128__)
	NORMAL_LOG_OCRA_SET(((lcb.period / 1000) * 16) - 1);
	NORMAL_LOG_TCNT_SET(69);
#endif
	NORMAL_LOG_ENABLE_INTERRUPT;
	LOG_STOP_INT_ENABLE;
	BUSY_LED_WAIT_TRIGGER_ON;
	lcb.buffPtr = lcb.buff;
	lcb.logCounter = lcb.buffSize - 1;
	lcb.busy = 1;
	while((((temp = LOGDATA_PIN) ^ lcb.TriggerPattern) & lcb.TriggerMask) != 0);	// wait for start
	NORMAL_LOG_TIMER_START;
	BUSY_LED_WAIT_TRIGGER_INV;

	*lcb.buffPtr++ = temp;
	set_sleep_mode(SLEEP_MODE_IDLE);
	if(lcb.logCounter < 10) lcb.dataReady = 1;
	while(!lcb.dataReady)
	{
		sleep_enable();
		sleep_cpu();
		sleep_disable();
	};
	lcb.busy = 0;
	NORMAL_LOG_TCCRB = 0;
	NORMAL_LOG_TIMSK = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//---------------------------------------------
//		^C}[rvݏ
//---------------------------------------------
ISR(TIMER1_COMPA_vect)
{
	*lcb.buffPtr++ = LOGDATA_PIN;
	--lcb.logCounter;
	if(lcb.logCounter <= 0)
	{
		lcb.dataReady = 1;
	}
	else
	{
		if(lcb.busyCounter++ >= lcb.busyCounterMax)
		{
			lcb.busyCounter = 0;
			BUSY_LED_WAIT_TRIGGER_INV;
		}
	}
}

////////////////////////////////////////////////////////////

//**********************************************************
//			Middle Speed Logging Function ( 10uS sampling )
//				200 Clock Cycle / 20MHz
//				160 Clock Cycle / 16MHz
//**********************************************************
void Log_200_Cycle()
{
	LOG_STOP_INT_ENABLE;
	BUSY_LED_WAIT_TRIGGER_ON;
	unsigned char temp;
	lcb.logCounter = lcb.buffSize - 2;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	lcb.busy = 1;
	while((((temp = LOGDATA_PIN) ^ lcb.TriggerPattern) & lcb.TriggerMask) != 0);	// wait for start

#if defined (__AVR_ATmega644P__)
	LOG_WAIT_186_CYCLE;
#endif
#if defined (__AVR_ATmega128__)
	LOG_WAIT_145_CYCLE;
#endif
	*buff++ = temp;
	do {
		LOG_DATA_1_BYTE_3_CYCLE(buff);
#if defined (__AVR_ATmega644P__)
		LOG_WAIT_174_CYCLE;
#endif
#if defined (__AVR_ATmega128__)
	LOG_WAIT_133_CYCLE;
#endif
	} while(--lcb.logCounter >= 0);
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//**********************************************************
//			Middle Speed Logging Function ( 5uS sampling )
//				100 Clock Cycle / 20MHz
//				 80 Clock Cycle / 16MHz
//**********************************************************
void Log_100_Cycle()
{
	LOG_STOP_INT_ENABLE;
	BUSY_LED_WAIT_TRIGGER_ON;
	unsigned char temp;
	lcb.logCounter = lcb.buffSize - 2;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	lcb.busy = 1;
	while((((temp = LOGDATA_PIN) ^ lcb.TriggerPattern) & lcb.TriggerMask) != 0);	// wait for start

#if defined (__AVR_ATmega644P__)
	LOG_WAIT_86_CYCLE;
#endif
#if defined (__AVR_ATmega128__)
	LOG_WAIT_65_CYCLE;
#endif
	*buff++ = temp;
	do {
		LOG_DATA_1_BYTE_3_CYCLE(buff);
#if defined (__AVR_ATmega644P__)
		LOG_WAIT_74_CYCLE;
#endif
#if defined (__AVR_ATmega128__)
	LOG_WAIT_53_CYCLE;
#endif
	} while(--lcb.logCounter >= 0);
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//**********************************************************
//			Middle Speed Logging Function ( 2uS sampling )
//				40 Clock Cycle / 20MHz
//				32 Clock Cycle / 16MHz
//**********************************************************
void Log_40_Cycle()
{
	LOG_STOP_INT_ENABLE;
	BUSY_LED_WAIT_TRIGGER_ON;
	unsigned char temp;
	lcb.logCounter = lcb.buffSize - 2;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	lcb.busy = 1;
	while((((temp = LOGDATA_PIN) ^ lcb.TriggerPattern) & lcb.TriggerMask) != 0);	// wait for start

#if defined (__AVR_ATmega644P__)
	LOG_WAIT_26_CYCLE;
#endif
#if defined (__AVR_ATmega128__)
	LOG_WAIT_18_CYCLE;
#endif

	*buff++ = temp;
	do {
		LOG_DATA_1_BYTE_3_CYCLE(buff);
#if defined (__AVR_ATmega644P__)
		LOG_WAIT_14_CYCLE;
#endif
#if defined (__AVR_ATmega128__)
    	LOG_WAIT_5_CYCLE;
#endif
	} while(--lcb.logCounter >= 0);
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//**********************************************************
//			Middle Speed Logging Function ( 1uS sampling )
//				20 Clock Cycle / 20MHz
//				16 Clock Cycle / 16MHz
//**********************************************************
void Log_20_Cycle()
{
	LOG_STOP_INT_ENABLE;
	BUSY_LED_WAIT_TRIGGER_ON;
	unsigned char temp;
	register volatile int logCounter asm("r26") = lcb.buffSize - 2;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	lcb.busy = 1;
	while((((temp = LOGDATA_PIN) ^ lcb.TriggerPattern) & lcb.TriggerMask) != 0);	// wait for start
#if defined (__AVR_ATmega644P__)
	LOG_WAIT_7_CYCLE;
#endif
#if defined (__AVR_ATmega128__)
   	LOG_WAIT_3_CYCLE;
#endif
	*buff++ = temp;
	do {
		LOG_DATA_1_BYTE_3_CYCLE(buff);
#if defined (__AVR_ATmega644P__)
		LOG_WAIT_13_CYCLE;
#endif
#if defined (__AVR_ATmega128__)
    	LOG_WAIT_8_CYCLE;
#endif
	} while(--logCounter >= 0);
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//**********************************************************
//			Middle Speed Logging Function ( 500nS sampling )
//				10 Clock Cycle / 20MHz
//				 8 Clock Cycle / 16MHz
//**********************************************************
void Log_10_Cycle()
{
	LOG_STOP_INT_ENABLE;
	BUSY_LED_WAIT_TRIGGER_ON;
	register volatile int logCounter asm("r26") = lcb.buffSize - 1;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	lcb.busy = 1;
	while((LOGDATA_PIN ^ lcb.TriggerPattern) & lcb.TriggerMask);	// wait for start
	do {
		LOG_DATA_1_BYTE_3_CYCLE(buff);
#if defined (__AVR_ATmega644P__)
		LOG_WAIT_3_CYCLE;
#endif
	} while(--logCounter >= 0);
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

//**********************************************************
//			Faster Logging Function
//				4 Clock Cycle
//**********************************************************
//			20MHz	0.2uS			5M samples/sec
//			16MHz	0.25uS			4M samples/sec

void Log_4_Cycle()
{
	LOG_STOP_INT_ENABLE;
	BUSY_LED_WAIT_TRIGGER_ON;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	lcb.busy = 1;
	while(((LOGDATA_PIN ^ lcb.TriggerPattern) & lcb.TriggerMask) != 0);	// wait for start
#if defined (__AVR_ATmega644P__)
	LOG_DATA_3_8K_BYTE_4(buff);
#endif
#if defined (__AVR_ATmega128__)
	LOG_DATA_30K_BYTE_3(buff);
#endif
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}

#if defined (__AVR_ATmega644P__)
//**********************************************************
//			Fastest Logging Function
//				3 Clock Cycle
//**********************************************************
//			20MHz	0.15uS			6.6M samples/sec

void Log_3_Cycle()
{
	LOG_STOP_INT_ENABLE;
	BUSY_LED_WAIT_TRIGGER_ON;
	register volatile unsigned char* buff asm("r30") = lcb.buff;
	lcb.busy = 1;
	while(((LOGDATA_PIN ^ lcb.TriggerPattern) & lcb.TriggerMask) != 0);	// wait for start
	LOG_DATA_3_8K_BYTE_3(buff);
	lcb.busy = 0;
	BUSY_LED_WAIT_TRIGGER_OFF;
	LOG_STOP_INT_DISABLE;
	return;
}
#endif
